<?php

namespace ePHPic\View;

use \Exception;

abstract class Attribute implements RenderableInterface
{
    protected mixed $value = null;
    public function __construct(mixed $value)
    {
        if(false === $this->validate($this->normalize($value)))
        {
            throw new Exception("Invalid Attribute value");
        }
        $this->value = $this->normalize($value);
    }
    public static function fromValue(mixed $value = null): static
    {
        return new static($value);
    }
    public function render(): string
    {
        if(true === $this->isEmptyAttribute())
        {
            return $this->__tostring();
        }
        if (null !== $this->value && "" !== $this->value && [] !== $this->value)
        {
            return $this->name() ."=\"". $this->__tostring() ."\"";
        }
        return "";
    }
    public function value(mixed $value = null): mixed
    {
        if(null !== $this->normalize($value))
        {
            if(false === $this->validate($value))
            {
                throw new Exception("Invalid Attribute value");
            }
            $this->value = $value;
        }
        return $this->value;
    }
    public function clear(): void
    {
        $this->value = null;
    }
    protected function normalize(mixed $value = null): mixed
    {
        return $value;
    }
    abstract protected function validate($value): bool;
    abstract public function __tostring(): string;
    abstract protected function name(): string;
    abstract protected function isEmptyAttribute(): bool;

}