<?php

namespace ePHPic\View;

use Exception;
use ReflectionClass;
use ReflectionNamedType;
use ReflectionProperty;

abstract class Component implements RenderableInterface
{

    abstract public function name(): string;

    abstract public function build(): Tag;

    public function __construct(array $properties = [])
    {
        foreach ($properties as $key => $value)
        {
            if (property_exists($this, $key))
            {
                $this->$key = $value;
            }
            else
            {
                throw new Exception("Undefined property $key for class " . __CLASS__);
            }
        }
    }

    public static function make(array $properties = []): static
    {
        return new static($properties);
    }

    public function render(): string
    {
        $container = $this->build();
        $container->setAttribute("component", $this->name());
        return $container->render();
    }
}